pub(crate) struct PhysicalFilterContext {
filter: Arc<dyn PhysicalExpr>,
expr_str: String,
column_id: ColumnId,
column_name: String,
semantic_type: SemanticType,
schema: SchemaRef,
immutable: bool,
}Expand description
Context to evaluate a physical expression for a parquet file.
Fields
filter: Arc<dyn PhysicalExpr>Filter to evaluate.
expr_str: StringDebug string of the original logical expression.
column_id: ColumnIdId of the column to evaluate.
column_name: StringName of the column to evaluate.
semantic_type: SemanticTypeSemantic type of the column.
schema: SchemaRefSchema containing only the referenced column.
immutable: boolWhether the original logical expression is immutable across queries.
Implementations
Sourceimpl PhysicalFilterContext
impl PhysicalFilterContext
Sourcepub(crate) fn new_opt(
sst_meta: &RegionMetadataRef,
expected_meta: Option<&RegionMetadata>,
read_format: &FlatReadFormat,
expr: &Expr,
) -> Option<Self>
pub(crate) fn new_opt( sst_meta: &RegionMetadataRef, expected_meta: Option<&RegionMetadata>, read_format: &FlatReadFormat, expr: &Expr, ) -> Option<Self>
Creates a context for the expr.
Returns None if the expression doesn’t reference exactly one column or the column to filter doesn’t exist in the SST metadata or the expected metadata.
Sourcefn is_prefilter_candidate(expr: &Expr) -> bool
fn is_prefilter_candidate(expr: &Expr) -> bool
Returns true if the expression is a variant we want to evaluate as a
physical prefilter. Binary exprs are intentionally excluded because
[SimpleFilterEvaluator] already handles them.
fn single_column_name(expr: &Expr) -> Option<String>
Sourcepub(crate) fn column_name(&self) -> &str
pub(crate) fn column_name(&self) -> &str
Returns the column name.
Sourcepub(crate) fn semantic_type(&self) -> SemanticType
pub(crate) fn semantic_type(&self) -> SemanticType
Returns the semantic type of the column.
Sourcepub(crate) fn schema(&self) -> &SchemaRef
pub(crate) fn schema(&self) -> &SchemaRef
Returns the schema containing only the referenced column.
Sourcepub(crate) fn is_immutable(&self) -> bool
pub(crate) fn is_immutable(&self) -> bool
Returns true if the original logical expression is immutable across queries.